@@ -1,7 +1,7 @@
 // qmidinet.cpp
 //
 /****************************************************************************
-   Copyright (C) 2010-2023, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2010-2024, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -29,6 +29,16 @@
 #include <QPainter>
 #include <QTimer>
 
+#ifdef CONFIG_XUNIQUE
+#include <QSharedMemory>
+#if QT_VERSION >= QT_VERSION_CHECK(6, 6, 0)
+#include <QNativeIpcKey>
+#endif
+#include <QLocalServer>
+#include <QLocalSocket>
+#include <QHostInfo>
+#endif	// CONFIG_XUNIQUE
+
 
 //-------------------------------------------------------------------------
 // qmidinetApplication -- Singleton application instance.
@@ -44,6 +54,10 @@ qmidinetApplication::qmidinetApplication ( int& argc, char **argv, bool bGUI )
 		, m_jack(this)
 	#endif
 		, m_udpd(this)
+	#ifdef CONFIG_XUNIQUE
+		, m_pMemory(nullptr)
+		, m_pServer(nullptr)
+	#endif
 {
 	if (bGUI) {
 	#if defined(Q_OS_LINUX) && !defined(CONFIG_WAYLAND)
@@ -60,9 +74,9 @@ qmidinetApplication::qmidinetApplication ( int& argc, char **argv, bool bGUI )
 		//	QMIDINET_TITLE " - " + QObject::tr(QMIDINET_SUBTITLE));
 	#if QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)
 		pApp->setDesktopFileName(
-			QString("org.rncbc.%1").arg(PACKAGE_TARNAME));
+			QString("org.rncbc.%1").arg(PROJECT_NAME));
 	#endif
-		QString sVersion(CONFIG_BUILD_VERSION);
+		QString sVersion(PROJECT_VERSION);
 		sVersion += '\n';
 		sVersion += QString("Qt: %1").arg(qVersion());
 	#if defined(QT_STATIC)
@@ -124,12 +138,151 @@ qmidinetApplication::qmidinetApplication ( int& argc, char **argv, bool bGUI )
 // Destructor.
 qmidinetApplication::~qmidinetApplication (void)
 {
+#ifdef CONFIG_XUNIQUE
+	clearServer();
+#endif	// CONFIG_XUNIQUE
+
 	if (m_pIcon) delete m_pIcon;
 	if (m_pApp)  delete m_pApp;
 }
 
 
-// Initializer.
+#ifdef CONFIG_XUNIQUE
+
+// Initializer (instance).
+bool qmidinetApplication::init (void)
+{
+	return setupServer();
+}
+
+
+// Local server/shmem setup.
+bool qmidinetApplication::setupServer (void)
+{
+	clearServer();
+
+	m_sUnique = m_pApp->applicationName();
+	QString sUserName = QString::fromUtf8(::getenv("USER"));
+	if (sUserName.isEmpty())
+		sUserName = QString::fromUtf8(::getenv("USERNAME"));
+	if (!sUserName.isEmpty()) {
+		m_sUnique += ':';
+		m_sUnique += sUserName;
+	}
+	m_sUnique += '@';
+	m_sUnique += QHostInfo::localHostName();
+#if QT_VERSION >= QT_VERSION_CHECK(6, 6, 0)
+	const QNativeIpcKey nativeKey
+		= QSharedMemory::legacyNativeKey(m_sUnique);
+#if defined(Q_OS_UNIX)
+	m_pMemory = new QSharedMemory(nativeKey);
+	m_pMemory->attach();
+	delete m_pMemory;
+#endif
+	m_pMemory = new QSharedMemory(nativeKey);
+#else
+#if defined(Q_OS_UNIX)
+	m_pMemory = new QSharedMemory(m_sUnique);
+	m_pMemory->attach();
+	delete m_pMemory;
+#endif
+	m_pMemory = new QSharedMemory(m_sUnique);
+#endif
+
+	bool bServer = false;
+	const qint64 pid = QCoreApplication::applicationPid();
+	struct Data { qint64 pid; };
+	if (m_pMemory->create(sizeof(Data))) {
+		m_pMemory->lock();
+		Data *pData = static_cast<Data *> (m_pMemory->data());
+		if (pData) {
+			pData->pid = pid;
+			bServer = true;
+		}
+		m_pMemory->unlock();
+	}
+	else
+	if (m_pMemory->attach()) {
+		m_pMemory->lock(); // maybe not necessary?
+		Data *pData = static_cast<Data *> (m_pMemory->data());
+		if (pData)
+			bServer = (pData->pid == pid);
+		m_pMemory->unlock();
+	}
+
+	if (bServer) {
+		QLocalServer::removeServer(m_sUnique);
+		m_pServer = new QLocalServer();
+		m_pServer->setSocketOptions(QLocalServer::UserAccessOption);
+		m_pServer->listen(m_sUnique);
+		QObject::connect(m_pServer,
+			 SIGNAL(newConnection()),
+			 SLOT(newConnectionSlot()));
+	} else {
+		QLocalSocket socket;
+		socket.connectToServer(m_sUnique);
+		if (socket.state() == QLocalSocket::ConnectingState)
+			socket.waitForConnected(200);
+		if (socket.state() == QLocalSocket::ConnectedState) {
+			socket.write(QCoreApplication::arguments().join(' ').toUtf8());
+			socket.flush();
+			socket.waitForBytesWritten(200);
+		}
+	}
+
+	return bServer;
+}
+
+
+// Local server/shmem cleanup.
+void qmidinetApplication::clearServer (void)
+{
+	if (m_pServer) {
+		m_pServer->close();
+		delete m_pServer;
+		m_pServer = nullptr;
+	}
+
+	if (m_pMemory) {
+		delete m_pMemory;
+		m_pMemory = nullptr;
+	}
+
+	m_sUnique.clear();
+}
+
+
+// Local server connection slot.
+void qmidinetApplication::newConnectionSlot (void)
+{
+	QLocalSocket *socket = m_pServer->nextPendingConnection();
+	QObject::connect(socket,
+		SIGNAL(readyRead()),
+		SLOT(readyReadSlot()));
+}
+
+
+// Local server data-ready slot.
+void qmidinetApplication::readyReadSlot (void)
+{
+	QLocalSocket *socket = qobject_cast<QLocalSocket *> (sender());
+	if (socket) {
+		const qint64 nread = socket->bytesAvailable();
+		if (nread > 0) {
+			const QByteArray data = socket->read(nread);
+			qmidinetOptions *pOptions = qmidinetOptions::getInstance();
+			if (pOptions && pOptions->parse_args(QString(data).split(' ')))
+				reset();
+			// Reset the server...
+			setupServer();
+		}
+	}
+}
+
+#endif	// CONFIG_XUNIQUE
+
+
+// Initializer (secondary).
 bool qmidinetApplication::setup (void)
 {
 	qmidinetOptions *pOptions = qmidinetOptions::getInstance();
@@ -335,7 +488,7 @@ void qmidinetSystemTrayIcon::about (void)
 	QString sText = "<h1>" QMIDINET_TITLE "</h1>\n";
 	sText += "<p>" + tr(QMIDINET_SUBTITLE) + "<br />\n";
 	sText += "<br />\n";
-	sText += tr("Version") + ": <b>" CONFIG_BUILD_VERSION "</b><br />\n";
+	sText += tr("Version") + ": <b>" PROJECT_VERSION "</b><br />\n";
 //	sText += "<small>" + tr("Build") + ": " CONFIG_BUILD_DATE "</small><br />\n";
 	if (!list.isEmpty()) {
 		sText += "<small><font color=\"red\">";
@@ -357,6 +510,7 @@ void qmidinetSystemTrayIcon::about (void)
 	sText += tr("This program is free software; you can redistribute it and/or modify it") + "<br />\n";
 	sText += tr("under the terms of the GNU General Public License version 2 or later.");
 	sText += "</small>";
+	sText += "<br />\n";
 	sText += "</p>\n";
 
 	QMessageBox abox;
@@ -445,6 +599,13 @@ int main ( int argc, char* argv[] )
 		return 1;
 	}
 
+#ifdef CONFIG_XUNIQUE
+	if (!app.init()) {
+		app.app()->quit();
+		return 2;
+	}
+#endif
+
 	app.reset();
 
 	return app.app()->exec();
